@@ -1,6 +1,7 @@
 /* slist.c -- generalised singly linked lists
 
-   Copyright (C) 2000, 2004, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2004, 2007-2009, 2011-2019, 2021-2022 Free
+   Software Foundation, Inc.
    Written by Gary V. Vaughan, 2000
 
    NOTE: The canonical source of this file is maintained with the
@@ -31,7 +32,7 @@ or obtained by writing to the Free Software Foundation, Inc.,
 #include <assert.h>
 
 #include "slist.h"
-#include <stddef.h>
+#include <stdlib.h>
 
 static SList *	slist_sort_merge    (SList *left, SList *right,
 				     SListCompare *compare, void *userdata);
@@ -73,7 +74,7 @@ slist_delete (SList *head, void (*delete_fct) (void *item))
            the stale item, you should probably return that from FIND if
 	   it makes a successful match.  Don't forget to slist_unbox()
 	   every item in a boxed list before operating on its contents.   */
-void *
+SList *
 slist_remove (SList **phead, SListCallback *find, void *matchdata)
 {
   SList *stale = 0;
@@ -107,7 +108,7 @@ slist_remove (SList **phead, SListCallback *find, void *matchdata)
 	}
     }
 
-  return result;
+  return (SList *) result;
 }
 
 /* Call FIND repeatedly with each element of SLIST and MATCHDATA, until
@@ -140,15 +141,18 @@ slist_find (SList *slist, SListCallback *find, void *matchdata)
 SList *
 slist_concat (SList *head, SList *tail)
 {
+  SList *last;
+
   if (!head)
     {
       return tail;
     }
 
-  while (head->next)
-    head = head->next;
+  last = head;
+  while (last->next)
+    last = last->next;
 
-  head->next = tail;
+  last->next = tail;
 
   return head;
 }
@@ -226,7 +230,7 @@ slist_reverse (SList *slist)
       next		= slist->next;
       slist->next	= result;
       result		= slist;
-      slist 		= next;
+      slist		= next;
     }
 
   return result;
@@ -311,6 +315,9 @@ slist_sort (SList *slist, SListCompare *compare, void *userdata)
   left = slist;
   right = slist->next;
 
+  if (!right)
+    return left;
+
   /* Skip two items with RIGHT and one with SLIST, until RIGHT falls off
      the end.  SLIST must be about half way along.  */
   while (right && (right = right->next))
@@ -338,7 +345,7 @@ slist_sort (SList *slist, SListCompare *compare, void *userdata)
    used for the boxes.  It us usually a very bad idea to mix boxed and
    non-boxed items in a single list.  */
 
-/* Return a `boxed' freshly mallocated 1 element list containing
+/* Return a 'boxed' freshly mallocated 1 element list containing
    USERDATA.  */
 SList *
 slist_box (const void *userdata)
@@ -354,7 +361,7 @@ slist_box (const void *userdata)
   return item;
 }
 
-/* Return the contents of a `boxed' ITEM, recycling the box itself.  */
+/* Return the contents of a 'boxed' ITEM, recycling the box itself.  */
 void *
 slist_unbox (SList *item)
 {
